import 'package:seeds/datasource/local/settings_storage.dart';
import 'package:seeds/datasource/remote/model/organization_model.dart';
import 'package:seeds/datasource/remote/model/profile_model.dart';
import 'package:seeds/domain-shared/page_state.dart';
import 'package:seeds/domain-shared/result_to_state_mapper.dart';
import 'package:seeds/i18n/profile_screens/profile/profile.i18n.dart';
import 'package:seeds/screens/profile_screens/contribution/interactor/viewmodels/scores_view_model.dart';
import 'package:seeds/screens/profile_screens/profile/interactor/viewmodels/profile_state.dart';

const int _profileResultIndex = 0;
const int _contributionScoreResultIndex = 1;
const int _communityScoreResultIndex = 2;
const int _reputationScoreResultIndex = 3;
const int _plantedScoreResultIndex = 4;
const int _transactionScoreResultIndex = 5;
const int _organizationResultIndex = 6;
const int _canResidentResultIndex = 7;
const int _canCitizenResultIndex = 8;

class ProfileValuesStateMapper extends StateMapper {
  ProfileState mapResultToState(ProfileState currentState, List<Result> results) {
    if (areAllResultsError(results)) {
      return currentState.copyWith(pageState: PageState.failure, errorMessage: 'Error Loading Page'.i18n);
    } else {
      // results.retainWhere((Result i) => i.isValue); // seems like a bug if there's 1 bad result it will do the wrong thing
      final ProfileModel? profile = results[_profileResultIndex].valueOrNull;
      final bool isCitizen = settingsStorage.isCitizen;
      final CitizenshipUpgradeStatus citizenshipUpgradeStatus;

      if (isCitizen) {
        final score = ScoresViewModel(
          contributionScore: results[_contributionScoreResultIndex].valueOrNull,
          communityScore: results[_communityScoreResultIndex].valueOrNull,
          reputationScore: results[_reputationScoreResultIndex].valueOrNull,
          plantedScore: results[_plantedScoreResultIndex].valueOrNull,
          transactionScore: results[_transactionScoreResultIndex].valueOrNull,
        );
        return currentState.copyWith(pageState: PageState.success, profile: profile, score: score);
      }

      final score = ScoresViewModel(
        contributionScore: results[_contributionScoreResultIndex].valueOrNull,
        communityScore: results[_communityScoreResultIndex].valueOrNull,
        reputationScore: results[_reputationScoreResultIndex].valueOrNull,
        plantedScore: results[_plantedScoreResultIndex].valueOrNull,
        transactionScore: results[_transactionScoreResultIndex].valueOrNull,
      );

      final organization = results[_organizationResultIndex].asValue?.value as List<OrganizationModel>;

      results[_canResidentResultIndex].isValue
          ? citizenshipUpgradeStatus = CitizenshipUpgradeStatus.canResident
          : results[_canCitizenResultIndex].isValue
              ? citizenshipUpgradeStatus = CitizenshipUpgradeStatus.canCitizen
              : citizenshipUpgradeStatus = CitizenshipUpgradeStatus.notReady;

      return currentState.copyWith(
        pageState: PageState.success,
        profile: profile,
        score: score,
        isOrganization: organization.isNotEmpty,
        citizenshipUpgradeStatus: citizenshipUpgradeStatus,
      );
    }
  }
}
